#include "mainwindow.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QContextMenuEvent>
#include <QMenu>
#include <QMenuBar>
#include <QFile>
#include <QApplication>
#include "canimal.h"
#include <QLabel>
#include <QBoxLayout>
#include <QLineEdit>
MainWindow::MainWindow(){
    saveAct = new QAction(tr("Save"), this);
    saveAct->setShortcuts(QKeySequence::Save);
    connect(saveAct, &QAction::triggered, this, &MainWindow::save);

    newAct = new QAction(tr("New"), this);
    newAct->setShortcuts(QKeySequence::New);
    connect(newAct, &QAction::triggered, this, &MainWindow::clear);

    exitAct = new QAction(tr("Exit"), this);
    exitAct->setShortcuts(QKeySequence::Quit);
    connect(exitAct, &QAction::triggered, this, &QWidget::close);

    aboutAct = new QAction(tr("About"), this);
    aboutAct->setShortcuts(QKeySequence::HelpContents);
    connect(aboutAct, &QAction::triggered, this, &MainWindow::about);

    aboutQtAct = new QAction(tr("About Qt"), this);
    connect(aboutQtAct, &QAction::triggered, qApp, &QApplication::aboutQt);

    fileMenu = menuBar()->addMenu(tr("File"));
    fileMenu->addAction(saveAct);
    fileMenu->addAction(newAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
    helpMenu = menuBar()->addMenu(tr("Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);
    setWindowTitle(tr("oop_lab7_prog"));
    QVBoxLayout *layout = new QVBoxLayout;

    QHBoxLayout *catLayout = new QHBoxLayout;
    QLabel *catLabel = new QLabel(tr("Cat:"), this);
    catLayout->addWidget(catLabel);
    catSpinBox = new QDoubleSpinBox(this);
    catSpinBox->setMinimum(0);
    catSpinBox->setMaximum(1000);
    catSpinBox->setValue(5);
    catSpinBox->setSingleStep(0.1);
    catSpinBox->setDecimals(1);
    catLayout->addWidget(catSpinBox);
    catLineEdit = new QLineEdit(this);

    QHBoxLayout *dogLayout = new QHBoxLayout;
    QLabel *dogLabel = new QLabel(tr("Dog:"), this);
    dogLayout->addWidget(dogLabel);
    dogSpinBox = new QDoubleSpinBox(this);
    dogSpinBox->setMinimum(0);
    dogSpinBox->setMaximum(1000);
    dogSpinBox->setValue(10);
    dogSpinBox->setSingleStep(0.2);
    dogSpinBox->setDecimals(1);
    dogLayout->addWidget(dogSpinBox);
    dogLineEdit = new QLineEdit(this);

    QHBoxLayout *horseLayout = new QHBoxLayout;
    QLabel *horseLabel = new QLabel(tr("Horse:"), this);
    horseLayout->addWidget(horseLabel);
    horseSpinBox = new QDoubleSpinBox(this);
    horseSpinBox->setMinimum(0);
    horseSpinBox->setMaximum(1000);
    horseSpinBox->setValue(400);
    horseLayout->addWidget(horseSpinBox);
    horseSpinBox->setDecimals(1);
    horseLineEdit = new QLineEdit(this);

    catLineEdit->setText("Frisky");
    dogLineEdit->setText("Bobik");
    horseLineEdit->setText("Juan");

    catLayout->addWidget(catLineEdit);
    layout->addLayout(catLayout);
    dogLayout->addWidget(dogLineEdit);
    layout->addLayout(dogLayout);
    horseLayout->addWidget(horseLineEdit);
    layout->addLayout(horseLayout);

    submitButton = new QPushButton(tr("Submit"), this);
    connect(submitButton, &QPushButton::clicked, this, &MainWindow::handleSubmit);
    layout->addWidget(submitButton);

    textEdit = new QTextEdit(this);
    layout->addWidget(textEdit);

    QWidget *centralWidget = new QWidget(this);
    centralWidget->setLayout(layout);
    setCentralWidget(centralWidget);
}

void MainWindow::handleSubmit() {
        CCat cat(fromLineEdit(catLineEdit), catSpinBox->value());
        CDog dog(fromLineEdit(dogLineEdit), dogSpinBox->value());
        CHorse horse(fromLineEdit(horseLineEdit), horseSpinBox->value());
        textEdit->append("Calories per day for " + QString::fromUtf8(cat.GetName()) + ": " + QString::number(cat.GetFoodCaloriesPerDay(),'f',0) + '\n' +
                         "Calories per day for " + QString::fromUtf8(dog.GetName()) + ": " + QString::number(dog.GetFoodCaloriesPerDay(),'f',0) + '\n' +
                         "Calories per day for " + QString::fromUtf8(horse.GetName()) + ": " + QString::number(horse.GetFoodCaloriesPerDay(),'f',0));    
        submitButton->setText("Append");
}

#ifndef QT_NO_CONTEXTMENU
void MainWindow::contextMenuEvent(QContextMenuEvent *event)
{
    QMenu menu(this);

    menu.addAction(saveAct);
    menu.addAction(newAct);
    menu.addAction(aboutAct);
    menu.exec(event->globalPos());
}
#endif // QT_NO_CONTEXTMENU

void MainWindow::save()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Зберігти файл"), "", tr("Текстові файли (*.txt)"));
    if (fileName.isEmpty()) return;
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        QMessageBox::warning(this, tr("Помилка"), tr("Файл неможливо відкрити для запису."));
        return;
    }

    QTextStream out(&file);
    out << textEdit->toPlainText();
    if (!textEdit->toPlainText().endsWith("\n"))out << '\n';

    file.close();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About oop_lab7_prog"),
                       tr("The <b>oop_lab7_prog</b> is a horrible example of laboratory work.<br>Created by Ivan."));
}

void MainWindow::clear()
{
    textEdit->clear();
    submitButton->setText("Submit");
}
